package org.linotte.frame.navinotte;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import javafx.application.Platform;
import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.concurrent.Worker.State;
import javafx.embed.swing.JFXPanel;
import javafx.event.EventHandler;
import javafx.scene.Scene;
import javafx.scene.web.WebEngine;
import javafx.scene.web.WebEvent;
import javafx.scene.web.WebView;

import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

import org.linotte.frame.Atelier;
import org.linotte.frame.navinotte.protocole.Explorer;
import org.linotte.frame.navinotte.protocole.Lire;
import org.linotte.frame.navinotte.protocole.Ouvrir;
import org.linotte.frame.navinotte.protocole.i.Protocole;
import org.linotte.moteur.outils.Ressources;
import org.linotte.moteur.xml.Version;
import org.linotte.moteur.xml.analyse.multilangage.Langage;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.events.EventListener;
import org.w3c.dom.events.EventTarget;

@SuppressWarnings("serial")
public class NavinotteJavaFX extends JPanel {

	private List<Protocole> handlers;

	private final JFXPanel jfxPanel = new JFXPanel();
	//private final JProgressBar progressBar = new JProgressBar();
	//private final JLabel lblStatus = new JLabel();
	private static WebEngine engine;
	private static JDialog navigateur;

	public static final String EVENT_TYPE_CLICK = "click";

	public NavinotteJavaFX(JDialog jd, Atelier atelier) {
		super(new BorderLayout());
		navigateur = jd;
		initComponents();
		add(jfxPanel, BorderLayout.CENTER);
		setPreferredSize(new Dimension(800, 500));

		// protocoles handlers :
		handlers = new ArrayList<Protocole>();
		handlers.add(new Lire(atelier));
		handlers.add(new Ouvrir());
		handlers.add(new Explorer(atelier));

	}

	private void initComponents() {
		createScene();
		/*JPanel statusBar = new JPanel(new BorderLayout(5, 0));
		statusBar.setBorder(BorderFactory.createEmptyBorder(3, 5, 3, 5));
		statusBar.add(lblStatus, BorderLayout.CENTER);
		statusBar.add(progressBar, BorderLayout.EAST);
		add(statusBar, BorderLayout.SOUTH);*/

	}

	private void createScene() {

		Platform.runLater(new Runnable() {
			@Override
			public void run() {

				WebView view = new WebView();
				engine = view.getEngine();

				engine.titleProperty().addListener(new ChangeListener<String>() {
					@Override
					public void changed(ObservableValue<? extends String> observable, String oldValue, final String newValue) {
						SwingUtilities.invokeLater(new Runnable() {
							@Override
							public void run() {
								navigateur.setTitle(newValue);
							}
						});
					}
				});

				engine.setOnStatusChanged(new EventHandler<WebEvent<String>>() {
					@Override
					public void handle(final WebEvent<String> event) {
						SwingUtilities.invokeLater(new Runnable() {
							@Override
							public void run() {
								//lblStatus.setText(event.getData());
							}
						});
					}
				});

				engine.locationProperty().addListener(new ChangeListener<String>() {
					@Override
					public void changed(ObservableValue<? extends String> ov, String oldValue, final String newValue) {
						SwingUtilities.invokeLater(new Runnable() {
							@Override
							public void run() {
								//txtURL.setText(newValue);
							}
						});
					}
				});

				engine.getLoadWorker().workDoneProperty().addListener(new ChangeListener<Number>() {
					@Override
					public void changed(ObservableValue<? extends Number> observableValue, Number oldValue, final Number newValue) {
						SwingUtilities.invokeLater(new Runnable() {
							@Override
							public void run() {
								//progressBar.setValue(newValue.intValue());
							}
						});
					}
				});

				/*engine.getLoadWorker().exceptionProperty().addListener(new ChangeListener<Throwable>() {

					public void changed(ObservableValue<? extends Throwable> o, Throwable old, final Throwable value) {
						if (engine.getLoadWorker().getState() == FAILED) {
							SwingUtilities.invokeLater(new Runnable() {
								@Override
								public void run() {
								}
							});
						}
					}
				});*/

				engine.getLoadWorker().stateProperty().addListener(new ChangeListener<State>() {
					@Override
					public void changed(ObservableValue ov, State oldState, State newState) {
						if (newState == State.SUCCEEDED) {
							EventListener listener = new EventListener() {
								@Override
								public void handleEvent(org.w3c.dom.events.Event ev) {
									String domEventType = ev.getType();
									if (domEventType.equals(EVENT_TYPE_CLICK)) {
										final String href = ((Element) ev.getTarget()).getAttribute("href");
										// Analyse du protocole
										for (final Protocole p : handlers) {
											if (p.can(href)) {
												Platform.runLater(new Runnable() {
													@Override
													public void run() {
														//System.out.println("1");
														p.handle(href);
														//System.out.println("2");
													}
												});

												return;
											}
										}
										System.out.println("go to " + href);
										loadURL(href);
									}
								}
							};

							Document doc = null;
							try {
								doc = engine.getDocument();
							} catch (Throwable e) {
								// TODO Auto-generated catch block
								e.printStackTrace();
							}
							NodeList nodeList = doc.getElementsByTagName("a");
							for (int i = 0; i < nodeList.getLength(); i++) {
								((EventTarget) nodeList.item(i)).addEventListener(EVENT_TYPE_CLICK, listener, false);
							}
						}
					}
				});
				jfxPanel.setScene(new Scene(view));
			}
		});
	}

	public static void loadURL(final String url) {
		if (navigateur != null) {
			Platform.runLater(new Runnable() {
				@Override
				public void run() {
					String tmp = toURL(url);

					if (tmp == null) {
						tmp = toURL("http://" + url);
					}

					engine.load(tmp);
				}
			});
		}
	}

	private static String toURL(String str) {
		try {
			return new URL(str).toExternalForm();
		} catch (MalformedURLException exception) {
			return null;
		}
	}

	public static void cacher() {
		navigateur.setVisible(false);
	}

	public static void ouvrir(Atelier atelier) {

		if (navigateur == null) {
			Langage langage = atelier.linotte.getLangage();
			navigateur = new JDialog(atelier, "Documentarium", false);
			navigateur.setSize(500, 500);
			navigateur.setDefaultCloseOperation(JDialog.HIDE_ON_CLOSE);
			navigateur.setIconImage(Ressources.getImageIcon("bandeau/help-content.png").getImage());
			NavinotteJavaFX browser = new NavinotteJavaFX(navigateur, atelier);
			navigateur.setContentPane(browser);
			//browser.loadURL("http://localhost:7777");
			loadURL(Version.getDocumentarium(langage));
			navigateur.pack();
			navigateur.setLocationRelativeTo(atelier);
		}
		navigateur.setVisible(true);
	}

}
