package org.linotte.greffons.ruby;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.jruby.embed.PathType;
import org.jruby.embed.ScriptingContainer;
import org.jruby.internal.runtime.methods.DynamicMethod;
import org.jruby.javasupport.JavaUtil;
import org.jruby.runtime.builtin.IRubyObject;
import org.linotte.greffons.GreffonsHandler;
import org.linotte.greffons.api.FabriqueGreffon;
import org.linotte.greffons.externe.Greffon;
import org.linotte.greffons.java.GreffonPrototype.Entrée;
import org.linotte.moteur.outils.Preference;
import org.linotte.moteur.outils.Ressources;
import org.linotte.moteur.xml.RegistreDesActions;

public class RubyFactory implements FabriqueGreffon {

	public static final String PREFIX = "slot_";

	private static ScriptingContainer container;

	private Greffon greffon;

	private String source;

	static {
		try {
			container = new ScriptingContainer();
			List<String> loadPaths = new ArrayList<String>();
			loadPaths.add(Ressources.getGreffons().getCanonicalPath());
			File repertoire_user = new File(Preference.getIntance().getHome() + File.separator + Preference.REPERTOIRE + File.separator
					+ GreffonsHandler.DIR_USER);
			loadPaths.add(repertoire_user.getCanonicalPath());
			loadPaths.add(Ressources.getGreffons().getCanonicalPath());
			container.setLoadPaths(loadPaths);
			container.setCurrentDirectory(Ressources.getGreffons().getCanonicalPath());
			System.out.println(container.getSupportedRubyVersion());
			RegistreDesActions.ajouterErreur("JRuby (Ruby) langage version : " + container.getSupportedRubyVersion());
		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	public RubyFactory() {
	}

	@Override
	public org.linotte.greffons.api.Greffon produire(String id, String classe) {
		try {

			RubyFactory engine = new RubyFactory(new File(Ressources.getGreffons().getCanonicalPath(), classe).getCanonicalPath());
			org.linotte.greffons.externe.Greffon o = engine.create();
			RubyGreffon greffon = new RubyGreffon(engine);
			greffon.setClass_(o.getClass());
			greffon.setId(id);
			greffon.setObjet(o);
			greffon.setSlots(engine.getSlots());
			return greffon;
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	private RubyFactory(String fichier) {
		source = fichier;
	}

	protected Greffon create() {
		greffon = (Greffon) container.runScriptlet(PathType.ABSOLUTE, source);
		return greffon;
	}

	private List<Entrée> getSlots() {
		List<Entrée> retour = new ArrayList<Entrée>();
		IRubyObject globValue = JavaUtil.convertJavaToRuby(container.getProvider().getRuntime(), greffon);
		Map<String, DynamicMethod> methods = globValue.getType().getMethods();
		for (Map.Entry<String, DynamicMethod> entrée : methods.entrySet()) {
			if (entrée.getKey().startsWith(PREFIX)) {
				retour.add(new Entrée(entrée.getKey().substring(PREFIX.length()), new RubyMethod(globValue, entrée.getKey())));
			}
		}
		return retour;
	}

}
